package data.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.PluginPick;
import com.fs.starfarer.api.characters.LevelBasedEffect;
import com.fs.starfarer.api.characters.ShipSkillEffect;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAIConfig;
import com.fs.starfarer.api.combat.ShipAIPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.ids.Personalities;
import com.fs.starfarer.rpg.Person;

public class sfc_titan {
    public static final float BASE_RANGE_FLAT_BONUS = 150f;
    public static final float WEAPON_FLUX_MULT = 0.75f;
    public static final float FIRE_RATE_MULT = 1.10f;
    public static final float SHIELD_DAMAGE_MULT = 0.90f;

    public static class Level1 implements ShipSkillEffect {

        public void apply(MutableShipStatsAPI stats, HullSize hullSize, String id, float level) {
            if (stats.getVariant() != null) {
                if (("sfciapetus".equals(stats.getVariant().getHullSpec().getHullId())) || ("sfcsuperiapetus".equals(stats.getVariant().getHullSpec().getHullId()))) {
                    stats.getEnergyWeaponRangeBonus().modifyFlat(id,BASE_RANGE_FLAT_BONUS);
                    stats.getBallisticWeaponRangeBonus().modifyFlat(id,BASE_RANGE_FLAT_BONUS);
                };
            }
        }

        public void unapply(MutableShipStatsAPI stats, HullSize hullSize, String id) {
            stats.getEnergyWeaponRangeBonus().unmodifyFlat(id);
            stats.getBallisticWeaponRangeBonus().unmodifyFlat(id);
        }

        public String getEffectDescription(float level) {
            return "Extends range of Ballistics and Energy weapons by " + (int)(BASE_RANGE_FLAT_BONUS) + " for Iapetus-Class ships.";
        }

        public String getEffectPerLevelDescription() {
            return null;
        }

        public ScopeDescription getScopeDescription() {
            return ScopeDescription.PILOTED_SHIP;
        }
    }

    public static class Level2 implements ShipSkillEffect {

        public void apply(MutableShipStatsAPI stats, HullSize hullSize, String id, float level) {
            if (stats.getVariant() != null) {
                if (("sfciapetus".equals(stats.getVariant().getHullSpec().getHullId())) || ("sfcsuperiapetus".equals(stats.getVariant().getHullSpec().getHullId()))) {
                    stats.getBallisticWeaponFluxCostMod().modifyMult(id, WEAPON_FLUX_MULT);
                    stats.getEnergyWeaponFluxCostMod().modifyMult(id, WEAPON_FLUX_MULT);
                };
            }
        }

        public void unapply(MutableShipStatsAPI stats, HullSize hullSize, String id) {
            stats.getBallisticWeaponFluxCostMod().unmodifyMult(id);
            stats.getEnergyWeaponFluxCostMod().unmodifyMult(id);
        }

        public String getEffectDescription(float level) {

            return "Decreases weapon flux fire cost for Ballistics and Energy weapons by 25% for Iapetus-class ships.";
        }

        public String getEffectPerLevelDescription() {
            return null;
        }

        public ScopeDescription getScopeDescription() {
            return ScopeDescription.PILOTED_SHIP;
        }
    }
    public static class Level3 implements ShipSkillEffect {

        public void apply(MutableShipStatsAPI stats, HullSize hullSize, String id, float level) {
            if (stats.getVariant() != null) {
                if (("sfciapetus".equals(stats.getVariant().getHullSpec().getHullId())) || ("sfcsuperiapetus".equals(stats.getVariant().getHullSpec().getHullId()))) {
                    stats.getBallisticRoFMult().modifyMult(id, FIRE_RATE_MULT);
                    stats.getEnergyRoFMult().modifyMult(id, FIRE_RATE_MULT);
                };
            }
        }

        public void unapply(MutableShipStatsAPI stats, HullSize hullSize, String id) {
            stats.getBallisticRoFMult().unmodifyMult(id);
            stats.getEnergyRoFMult().unmodifyMult(id);
        }

        public String getEffectDescription(float level) {

            return "Increases fire rate of Ballistic and Energy weapons by 10% for Iapetus-class ships.";
        }

        public String getEffectPerLevelDescription() {
            return null;
        }

        public ScopeDescription getScopeDescription() {
            return ScopeDescription.PILOTED_SHIP;
        }
    }

    public static class Level4 implements ShipSkillEffect {

        public void apply(MutableShipStatsAPI stats, HullSize hullSize, String id, float level) {
            if (stats.getVariant() != null) {
                if (("sfciapetus".equals(stats.getVariant().getHullSpec().getHullId())) || ("sfcsuperiapetus".equals(stats.getVariant().getHullSpec().getHullId()))) {
                    stats.getKineticShieldDamageTakenMult().modifyMult(id, SHIELD_DAMAGE_MULT);
                };
            }

        }

        public void unapply(MutableShipStatsAPI stats, HullSize hullSize, String id) {
            stats.getKineticShieldDamageTakenMult().unmodifyMult(id);
        }

        public String getEffectDescription(float level) {

            return "Decreases kinetic shield damage by 10% for Iapetus-class ships.";
        }

        public String getEffectPerLevelDescription() {
            return null;
        }

        public ScopeDescription getScopeDescription() {
            return ScopeDescription.PILOTED_SHIP;
        }
    }

  /*  public static class Level5 implements ShipSkillEffect {

        public void apply(ShipAIPlugin pickShipAI(FleetMemberAPI member, ShipAPI ship)) {
            ShipAPI ship = null;
            boolean iapetus = true;
            if (ship.getVariant() != null) {
                iapetus = (("sfciapetus".equals(ship.getVariant().getHullSpec().getHullId())) || ("sfcsuperiapetus".equals(ship.getVariant().getHullSpec().getHullId())));
            }
            if (iapetus) {
                ship.getShipAI().getConfig().personalityOverride = Personalities.AGGRESSIVE;
            } else {
                ship.getShipAI().getConfig().personalityOverride = Personalities.TIMID;
            }
        }
        public void unapply(ShipAIPlugin pickShipAI(FleetMemberAPI member, ShipAPI ship)) {
            ship.getShipAI().getConfig().personalityOverride = unapply();
        }

        public String getEffectDescription(float level) {
            return "Forces aggressive personality when piloting Iapetus-class ships.";
        }

        public String getEffectPerLevelDescription() {
            return null;
        }

        public ScopeDescription getScopeDescription() {
            return ScopeDescription.PILOTED_SHIP;
        }
    }*/
}
